# This file has been generated


build_system = 'AppVeyor'
build_id = '30440598'
build_number = '453'
build_version = '1.0.453'
job_id = 'uxgqfytggh822ud5'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/stlib/build/job/uxgqfytggh822ud5'
